@@ -6,27 +6,37 @@
 import os
 import urllib
 import urllib.request
-from datetime import datetime
+from datetime import date, datetime
 from time import sleep
 from typing import Dict, List, Optional, Tuple, Union
 
-from here_location_services.config.routing_config import PlaceOptions, Scooter, WayPointOptions
+from geojson import LineString, MultiPolygon, Point, Polygon
 
+from here_location_services.config.routing_config import Scooter, Via
+from here_location_services.platform.apis.aaa_oauth2_api import AAAOauth2Api
+from here_location_services.platform.auth import Auth
+from here_location_services.platform.credentials import PlatformCredentials
+
+from .autosuggest_api import AutosuggestApi
+from .config.autosuggest_config import SearchCircle
+from .config.base_config import PlaceOptions, Truck, WayPointOptions
 from .config.matrix_routing_config import (
     AutoCircleRegion,
     AvoidBoundingBox,
     BoundingBoxRegion,
     CircleRegion,
     PolygonRegion,
-    Truck,
     WorldRegion,
 )
+from .destination_weather_api import DestinationWeatherApi
 from .exceptions import ApiError
 from .geocoding_search_api import GeocodingSearchApi
 from .isoline_routing_api import IsolineRoutingApi
 from .matrix_routing_api import MatrixRoutingApi
 from .responses import (
+    AutosuggestResponse,
     BrowseResponse,
+    DestinationWeatherResponse,
     DiscoverResponse,
     GeocoderResponse,
     IsolineResponse,
@@ -47,29 +57,52 @@ class LS:
     def __init__(
         self,
         api_key: Optional[str] = None,
+        platform_credentials: Optional[PlatformCredentials] = None,
         proxies: Optional[dict] = None,
         country: str = "row",
     ):
         api_key = api_key or os.environ.get("LS_API_KEY")
-        self.credentials = dict(api_key=api_key)
+        self.auth: Optional[Auth] = None
+        if not api_key:
+            credentials = platform_credentials or PlatformCredentials.from_default()
+            aaa_oauth2_api = AAAOauth2Api(
+                base_url=credentials.cred_properties["endpoint"], proxies={}
+            )
+            self.auth = Auth(credentials=credentials, aaa_oauth2_api=aaa_oauth2_api)
+
         self.proxies = proxies or urllib.request.getproxies()
         self.geo_search_api = GeocodingSearchApi(
             api_key=api_key,
+            auth=self.auth,
             proxies=proxies,
             country=country,
         )
         self.isoline_routing_api = IsolineRoutingApi(
             api_key=api_key,
+            auth=self.auth,
             proxies=proxies,
             country=country,
         )
         self.routing_api = RoutingApi(
             api_key=api_key,
+            auth=self.auth,
             proxies=proxies,
             country=country,
         )
         self.matrix_routing_api = MatrixRoutingApi(
-            api_key=api_key, proxies=proxies, country=country
+            api_key=api_key, auth=self.auth, proxies=proxies, country=country
+        )
+        self.autosuggest_api = AutosuggestApi(
+            api_key=api_key,
+            auth=self.auth,
+            proxies=proxies,
+            country=country,
+        )
+        self.destination_weather_api = DestinationWeatherApi(
+            api_key=api_key,
+            auth=self.auth,
+            proxies=proxies,
+            country=country,
         )
 
     def geocode(self, query: str, limit: int = 20, lang: str = "en-US") -> GeocoderResponse:
@@ -116,13 +149,22 @@ class LS:
 
     def calculate_isoline(
         self,
-        mode: str,
         range: str,
         range_type: str,
-        start: Optional[List[float]] = None,
-        destination: Optional[List[float]] = None,
-        arrival: Optional[str] = None,
-        departure: Optional[str] = None,
+        transport_mode: str,
+        origin: Optional[List] = None,
+        departure_time: Optional[datetime] = None,
+        destination: Optional[List] = None,
+        arrival_time: Optional[datetime] = None,
+        routing_mode: Optional[str] = "fast",
+        shape_max_points: Optional[int] = None,
+        optimised_for: Optional[str] = "balanced",
+        avoid_features: Optional[List[str]] = None,
+        truck: Optional[Truck] = None,
+        origin_place_options: Optional[PlaceOptions] = None,
+        origin_waypoint_options: Optional[WayPointOptions] = None,
+        destination_place_options: Optional[PlaceOptions] = None,
+        destination_waypoint_options: Optional[WayPointOptions] = None,
     ) -> IsolineResponse:
         """Calculate isoline routing.
 
@@ -130,53 +172,247 @@ class LS:
         leaving from one defined center with either a specified length
         or specified travel time.
 
-        :param mode: A string representing how the route is calculated.
-            Example: ``Type;TransportModes;TrafficMode;Feature``.
-            ``fastest;car;traffic:disabled;motorway:-3``
         :param range: A string representing a range of isoline, unit is defined by
             parameter range type. Example: range='1000' or range='1000,2000,3000'
-        :param range_type: A string representing a type of `range`. Possible values are
+        :param range_type: A string representing a type of ``range``. Possible values are
             ``distance``, ``time`` and ``consumption``. For distance the unit meters. For a
-            time the unit is seconds.For consumption, it is defined by the consumption
+            time the unit is seconds. For consumption, it is defined by the consumption
             model.
-        :param start: A list of latitude and longitude representing the center of isoline
-            request. Isoline will cover all the roads which can be reached from this
-            point within a given range. It can not be used in combination with the
-            ``destination`` parameter.
-        :param destination: A list of latitude and longitude representing the center of
-            isoline request. Isoline will cover all roads from which this point can be
-            reached within a given range. It can not be used in combination with the
-            ``start`` parameter.
-        :param arrival: A string representing the time when travel is expected to end.
-            It can be used only if the parameter ``destination`` is also used.
-            Example: arrival= '2013-07-04T17:00:00+02'.
-        :param departure: A string representing the time when travel is expected to
-            start. It can be used only if the parameter ``start`` is also used.
-            Example: departure= '2013-07-04T17:00:00+02'
-        :raises ValueError: If ``start`` and ``destination`` are provided togrther.
+        :param transport_mode: A string representing Mode of transport to be used for the
+            calculation of the isolines.
+            Example: ``car``.
+        :param origin: Center of the isoline request. The Isoline(s) will cover the region
+            which can be reached from this point within given range. It cannot be used in
+            combination with ``destination`` parameter.
+        :param departure_time: Specifies the time of departure as defined by either date-time
+            or full-date partial-time in RFC 3339, section 5.6 (for example, 2019-06-24T01:23:45).
+            The requested time is converted to the local time at origin. When the optional timezone
+            offset is not specified, time is assumed to be local. If neither departure_time or
+            arrival_time are specified, current time at departure location will be used. All Time
+            values in the response are returned in the timezone of each location.
+        :param destination: Center of the isoline request. The Isoline(s) will cover the
+            region within the specified range that can reach this point. It cannot be used
+            in combination with ``origin`` parameter.
+        :param arrival_time: Specifies the time of arrival as defined by either date-time or
+            full-date T partial-time in RFC 3339, section 5.6 (for example, 2019-06-24T01:23:45).
+            The requested time is converted to the local time at destination. When the optional
+            timezone offset is not specified, time is assumed to be local. All Time values in
+            the response are returned in the timezone of each location.
+        :param routing_mode: A string to represent routing mode.
+        :param shape_max_points: An integer to Limit the number of points in the resulting isoline
+            geometry. If the isoline consists of multiple components, the sum of points from all
+            components is considered. This parameter doesn't affect performance.
+        :param optimised_for: A string to specify how isoline calculation is optimized.
+        :param avoid_features: Avoid routes that violate these properties. Avoid features
+            are defined in :attr:
+            `AVOID_FEATURES <here_location_services.config.isoline_routing_config.AVOID_FEATURES>`
+        :param truck: Different truck options to use during route calculation when transport_mode
+            = truck. use object of :class:`Truck here_location_services.config.base_config.Truck>`
+        :param origin_place_options: :class:`PlaceOptions` optinal place options for ``origin``.
+        :param origin_waypoint_options: :class:`WayPointOptions` optional waypoint options
+            for ``origin``.
+        :param destination_place_options: :class:`PlaceOptions` optinal place options
+            for ``destination``.
+        :param destination_waypoint_options: :class:`WayPointOptions` optional waypoint options
+            for ``destination``.
+        :raises ValueError: If ``origin`` and ``destination`` are provided together.
         :return: :class:`IsolineResponse` object.
         """
 
-        if start and destination:
-            raise ValueError("`start` and `destination` can not be provided together.")
-        if start is None and destination is None:
-            raise ValueError("please provide either `start` or `destination`.")
-        if departure and start is None:
-            raise ValueError("`departure` must be provided with `start`")
-        if arrival and destination is None:
+        if origin and destination:
+            raise ValueError("`origin` and `destination` can not be provided together.")
+        if origin is None and destination is None:
+            raise ValueError("please provide either `origin` or `destination`.")
+        if departure_time and origin is None:
+            raise ValueError("`departure_time` must be provided with `origin`")
+        if arrival_time and destination is None:
             raise ValueError("`arrival` must be provided with `destination`")
 
         resp = self.isoline_routing_api.get_isoline_routing(
-            mode=mode,
             range=range,
             range_type=range_type,
-            start=start,
+            transport_mode=transport_mode,
+            origin=origin,
+            departure_time=departure_time,
             destination=destination,
-            arrival=arrival,
-            departure=departure,
+            arrival_time=arrival_time,
+            routing_mode=routing_mode,
+            shape_max_points=shape_max_points,
+            optimised_for=optimised_for,
+            avoid_features=avoid_features,
+            truck=truck,
+            origin_place_options=origin_place_options,
+            origin_waypoint_options=origin_waypoint_options,
+            destination_place_options=destination_place_options,
+            destination_waypoint_options=destination_waypoint_options,
         )
-        response = resp.json()["response"]
-        return IsolineResponse.new(response)
+        response = IsolineResponse.new(resp.json())
+
+        if response.notices:
+            raise ValueError("Isolines could not be calculated.")
+        return response
+
+    def autosuggest(
+        self,
+        query: str,
+        at: Optional[List] = None,
+        search_in_circle: Optional[SearchCircle] = None,
+        search_in_bbox: Optional[Tuple] = None,
+        in_country: Optional[List[str]] = None,
+        limit: Optional[int] = 20,
+        terms_limit: Optional[int] = None,
+        lang: Optional[List[str]] = None,
+        political_view: Optional[str] = None,
+        show: Optional[List[str]] = None,
+    ) -> AutosuggestResponse:
+        """Suggest address or place candidates based on an incomplete or misspelled query
+
+        :param query: A string for free-text query. Example: res, rest
+        :param at: Specify the center of the search context expressed as list of coordinates
+            One of `at`, `search_in_circle` or `search_in_bbox` is required.
+            Parameters "at", "search_in_circle" and "search_in_bbox" are mutually exclusive. Only
+            one of them is allowed.
+        :param search_in_circle: Search within a circular geographic area provided as
+            latitude, longitude, and radius (in meters)
+        :param search_in_bbox: Search within a rectangular bounding box geographic area provided
+            as tuple of west longitude, south latitude, east longitude, north latitude
+        :param in_country: Search within a specific or multiple countries provided as
+            comma-separated ISO 3166-1 alpha-3 country codes. The country codes are to be
+            provided in all uppercase. Must be accompanied by exactly one of
+            `at`, `search_in_circle` or `search_in_bbox`.
+        :param limit: An integer specifiying maximum number of results to be returned.
+        :param terms_limit: An integer specifiying maximum number of Query Terms Suggestions
+            to be returned.
+        :param lang: List of strings to select the language to be used for result rendering
+            from a list of BCP 47 compliant language codes.
+        :param political_view: Toggle the political view.
+        :param show: Select additional fields to be rendered in the response. Please note
+            that some of the fields involve additional webservice calls and can increase
+            the overall response time.
+        :return: :class:`requests.Response` object.
+        :raises ValueError: If ``search_in_circle``,``search_in_bbox`` and ``destination``
+            are provided together.
+        """
+
+        i = iter([search_in_circle, search_in_bbox, at])
+        if not (any(i) and not any(i)):
+            raise ValueError(
+                "Exactly one of `search_in_circle` or `search_in_bbox` or `at` must be provided."
+            )
+
+        resp = self.autosuggest_api.get_autosuggest(
+            query=query,
+            at=at,
+            search_in_bbox=search_in_bbox,
+            search_in_circle=search_in_circle,
+            in_country=in_country,
+            limit=limit,
+            terms_limit=terms_limit,
+            lang=lang,
+            political_view=political_view,
+            show=show,
+        )
+        response = AutosuggestResponse.new(resp.json())
+
+        return response
+
+    def get_dest_weather(
+        self,
+        products: List[str],
+        at: Optional[List] = None,
+        query: Optional[str] = None,
+        zipcode: Optional[str] = None,
+        hourly_date: Optional[Union[date, datetime]] = None,
+        one_observation: Optional[bool] = None,
+        language: Optional[str] = None,
+        units: Optional[str] = None,
+    ) -> DestinationWeatherResponse:
+        """Retrieves weather reports, weather forecasts, severe weather alerts
+            and moon and sun rise and set information.
+
+        :param products: List of :class:`DestWeatherProduct` identifying the type of
+            report to obtain.
+        :param at: A list of ``latitude`` and ``longitude`` specifying the area covered
+            by the weather report.
+        :param query: Free text query. Examples: "125, Berliner, berlin", "Beacon, Boston"
+        :param zipcode: ZIP code of the location. This parameter is supported only for locations in
+            the United States of America.
+        :param hourly_date: Date for which hourly forecasts are to be retrieved. Can be either a
+            `date` or `datetime` object
+        :param one_observation: Boolean, if set to true, the response only includes the closest
+            location. Only available when the `product` parameter is set to
+            `DEST_WEATHER_PRODUCT.observation`.
+        :param language: Defines the language used in the descriptions in the response.
+        :param units: Defines whether units or imperial units are used in the response.
+        :raises ValueError: If neither `at`, `query` or `zipcode` are passed.
+        :raises ValueError: If `one_observation` is set to true without passing
+            DEST_WEATHER_PRODUCT.observation in `products`
+        :return: :class:`DestinationWeatherResponse` object.
+        """
+
+        if at is None and query is None and zipcode is None:
+            raise ValueError("please provide either `at` or `query` or `zipcode`.")
+        if "observation" not in products and one_observation:
+            raise ValueError(
+                "`one_observation` can only be set when the `products` parameter "
+                + "is set to DEST_WEATHER_PRODUCT.observation"
+            )
+
+        resp = self.destination_weather_api.get_dest_weather(
+            products=products,
+            at=at,
+            query=query,
+            zipcode=zipcode,
+            hourly_date=hourly_date,
+            one_observation=one_observation,
+            language=language,
+            units=units,
+        )
+        response = DestinationWeatherResponse.new(resp.json())
+        return response
+
+    def get_weather_alerts(
+        self,
+        geometry: Union[Point, LineString, Polygon, MultiPolygon],
+        start_time: datetime,
+        id: Optional[str] = None,
+        weather_severity: Optional[int] = None,
+        weather_type: Optional[str] = None,
+        country: Optional[str] = None,
+        end_time: Optional[datetime] = None,
+        width: Optional[int] = 50000,
+    ) -> DestinationWeatherResponse:
+        """Retrieves weather reports, weather forecasts, severe weather alerts
+            and moon and sun rise and set information.
+
+        :param geometry: Point or LineString or Polygon or MultiPolygon defining the route or
+            a single location
+        :param start_time: Start time of the event
+        :param id: Unique weather alert id.
+        :param weather_severity: Defines the severity of the weather event as defined
+            in :class:`WeatherSeverity`.
+        :param weather_type: Defines the type of the weather event as defined
+            in :class:`WeatherType`.
+        :param country: String for ISO-3166-1 2-letter country code.
+        :param end_time: End time of the event. If not present, warning is valid until
+            it is not removed from the feed by national weather institutes
+            (valid until warning is present in the response)
+        :param width: int. default 50000
+        :return: :class:`DestinationWeatherResponse` object.
+        """
+
+        resp = self.destination_weather_api.get_weather_alerts(
+            geometry=geometry,
+            id=id,
+            weather_severity=weather_severity,
+            weather_type=weather_type,
+            country=country,
+            start_time=start_time,
+            end_time=end_time,
+            width=width,
+        )
+        response = DestinationWeatherResponse.new(resp.json())
+        return response
 
     def discover(
         self,
@@ -283,12 +519,11 @@ class LS:
         self,
         origin: List,
         destination: List,
-        via: Optional[List[Tuple]] = None,
+        via: Optional[List[Via]] = None,
         origin_place_options: Optional[PlaceOptions] = None,
+        origin_waypoint_options: Optional[WayPointOptions] = None,
         destination_place_options: Optional[PlaceOptions] = None,
-        via_place_options: Optional[PlaceOptions] = None,
         destination_waypoint_options: Optional[WayPointOptions] = None,
-        via_waypoint_options: Optional[WayPointOptions] = None,
         departure_time: Optional[datetime] = None,
         routing_mode: str = "fast",
         alternatives: int = 0,
@@ -304,15 +539,14 @@ class LS:
 
         :param origin: A list of ``latitude`` and ``longitude`` of origin point of route.
         :param destination: A list of ``latitude`` and ``longitude`` of destination point of route.
-        :param via: A list of tuples of ``latitude`` and ``longitude`` of via points.
+        :param via: A list of :class:`Via` objects.
         :param origin_place_options: :class:`PlaceOptions` optinal place options for ``origin``.
+        :param origin_waypoint_options: :class:`WayPointOptions` optional waypoint options
+            for ``origin``.
         :param destination_place_options: :class:`PlaceOptions` optinal place options
             for ``destination``.
-        :param via_place_options: :class:`PlaceOptions` optinal place options for ``via``.
         :param destination_waypoint_options: :class:`WayPointOptions` optional waypoint options
             for ``destination``.
-        :param via_waypoint_options: :class:`WayPointOptions` optional waypoint options for
-            ``via``.
         :param departure_time: :class:`datetime.datetime` object.
         :param routing_mode: A string to represent routing mode. use config defined in :attr:`ROUTING_MODE <here_location_services.config.routing_config.ROUTING_MODE>`
         :param alternatives: Number of alternative routes to return aside from the optimal route.
@@ -331,16 +565,16 @@ class LS:
             (ISO-3166-1 alpha-3 code) that routes will exclude.
         :return: :class:`RoutingResponse` object.
         """  # noqa: E501
+
         resp = self.routing_api.route(
             transport_mode="car",
             origin=origin,
             destination=destination,
             via=via,
             origin_place_options=origin_place_options,
+            origin_waypoint_options=origin_waypoint_options,
             destination_place_options=destination_place_options,
-            via_place_options=via_place_options,
             destination_waypoint_options=destination_waypoint_options,
-            via_waypoint_options=via_waypoint_options,
             departure_time=departure_time,
             routing_mode=routing_mode,
             alternatives=alternatives,
@@ -358,12 +592,11 @@ class LS:
         self,
         origin: List,
         destination: List,
-        via: Optional[List[Tuple]] = None,
+        via: Optional[List[Via]] = None,
         origin_place_options: Optional[PlaceOptions] = None,
+        origin_waypoint_options: Optional[WayPointOptions] = None,
         destination_place_options: Optional[PlaceOptions] = None,
-        via_place_options: Optional[PlaceOptions] = None,
         destination_waypoint_options: Optional[WayPointOptions] = None,
-        via_waypoint_options: Optional[WayPointOptions] = None,
         departure_time: Optional[datetime] = None,
         routing_mode: str = "fast",
         alternatives: int = 0,
@@ -379,15 +612,14 @@ class LS:
 
         :param origin: A list of ``latitude`` and ``longitude`` of origin point of route.
         :param destination: A list of ``latitude`` and ``longitude`` of destination point of route.
-        :param via: A list of tuples of ``latitude`` and ``longitude`` of via points.
+        :param via: A list of :class:`Via` objects.
         :param origin_place_options: :class:`PlaceOptions` optinal place options for ``origin``.
+        :param origin_waypoint_options: :class:`WayPointOptions` optional waypoint options
+            for ``origin``.
         :param destination_place_options: :class:`PlaceOptions` optinal place options
             for ``destination``.
-        :param via_place_options: :class:`PlaceOptions` optinal place options for ``via``.
         :param destination_waypoint_options: :class:`WayPointOptions` optional waypoint options
             for ``destination``.
-        :param via_waypoint_options: :class:`WayPointOptions` optional waypoint options for
-            ``via``.
         :param departure_time: :class:`datetime.datetime` object.
         :param routing_mode: A string to represent routing mode.
         :param alternatives: Number of alternative routes to return aside from the optimal route.
@@ -412,10 +644,9 @@ class LS:
             destination=destination,
             via=via,
             origin_place_options=origin_place_options,
+            origin_waypoint_options=origin_waypoint_options,
             destination_place_options=destination_place_options,
-            via_place_options=via_place_options,
             destination_waypoint_options=destination_waypoint_options,
-            via_waypoint_options=via_waypoint_options,
             departure_time=departure_time,
             routing_mode=routing_mode,
             alternatives=alternatives,
@@ -433,12 +664,11 @@ class LS:
         self,
         origin: List,
         destination: List,
-        via: Optional[List[Tuple]] = None,
+        via: Optional[List[Via]] = None,
         origin_place_options: Optional[PlaceOptions] = None,
+        origin_waypoint_options: Optional[WayPointOptions] = None,
         destination_place_options: Optional[PlaceOptions] = None,
-        via_place_options: Optional[PlaceOptions] = None,
         destination_waypoint_options: Optional[WayPointOptions] = None,
-        via_waypoint_options: Optional[WayPointOptions] = None,
         departure_time: Optional[datetime] = None,
         routing_mode: str = "fast",
         alternatives: int = 0,
@@ -455,15 +685,14 @@ class LS:
 
         :param origin: A list of ``latitude`` and ``longitude`` of origin point of route.
         :param destination: A list of ``latitude`` and ``longitude`` of destination point of route.
-        :param via: A list of tuples of ``latitude`` and ``longitude`` of via points.
+        :param via: A list of :class:`Via` objects.
         :param origin_place_options: :class:`PlaceOptions` optinal place options for ``origin``.
+        :param origin_waypoint_options: :class:`WayPointOptions` optional waypoint options
+            for ``origin``.
         :param destination_place_options: :class:`PlaceOptions` optinal place options
             for ``destination``.
-        :param via_place_options: :class:`PlaceOptions` optinal place options for ``via``.
         :param destination_waypoint_options: :class:`WayPointOptions` optional waypoint options
             for ``destination``.
-        :param via_waypoint_options: :class:`WayPointOptions` optional waypoint options for
-            ``via``.
         :param departure_time: :class:`datetime.datetime` object.
         :param routing_mode: A string to represent routing mode.
         :param alternatives: Number of alternative routes to return aside from the optimal route.
@@ -490,10 +719,9 @@ class LS:
             destination=destination,
             via=via,
             origin_place_options=origin_place_options,
+            origin_waypoint_options=origin_waypoint_options,
             destination_place_options=destination_place_options,
-            via_place_options=via_place_options,
             destination_waypoint_options=destination_waypoint_options,
-            via_waypoint_options=via_waypoint_options,
             departure_time=departure_time,
             routing_mode=routing_mode,
             alternatives=alternatives,
@@ -512,12 +740,11 @@ class LS:
         self,
         origin: List,
         destination: List,
-        via: Optional[List[Tuple]] = None,
+        via: Optional[List[Via]] = None,
         origin_place_options: Optional[PlaceOptions] = None,
+        origin_waypoint_options: Optional[WayPointOptions] = None,
         destination_place_options: Optional[PlaceOptions] = None,
-        via_place_options: Optional[PlaceOptions] = None,
         destination_waypoint_options: Optional[WayPointOptions] = None,
-        via_waypoint_options: Optional[WayPointOptions] = None,
         scooter: Optional[Scooter] = None,
         departure_time: Optional[datetime] = None,
         routing_mode: str = "fast",
@@ -534,15 +761,14 @@ class LS:
 
         :param origin: A list of ``latitude`` and ``longitude`` of origin point of route.
         :param destination: A list of ``latitude`` and ``longitude`` of destination point of route.
-        :param via: A list of tuples of ``latitude`` and ``longitude`` of via points.
+        :param via: A list of :class:`Via` objects.
         :param origin_place_options: :class:`PlaceOptions` optinal place options for ``origin``.
+        :param origin_waypoint_options: :class:`WayPointOptions` optional waypoint options
+            for ``origin``.
         :param destination_place_options: :class:`PlaceOptions` optinal place options
             for ``destination``.
-        :param via_place_options: :class:`PlaceOptions` optinal place options for ``via``.
         :param destination_waypoint_options: :class:`WayPointOptions` optional waypoint options
             for ``destination``.
-        :param via_waypoint_options: :class:`WayPointOptions` optional waypoint options for
-            ``via``.
         :param scooter: Additional attributes for scooter route.
         :param departure_time: :class:`datetime.datetime` object.
         :param routing_mode: A string to represent routing mode.
@@ -568,10 +794,9 @@ class LS:
             destination=destination,
             via=via,
             origin_place_options=origin_place_options,
+            origin_waypoint_options=origin_waypoint_options,
             destination_place_options=destination_place_options,
-            via_place_options=via_place_options,
             destination_waypoint_options=destination_waypoint_options,
-            via_waypoint_options=via_waypoint_options,
             scooter=scooter,
             departure_time=departure_time,
             routing_mode=routing_mode,
@@ -590,12 +815,11 @@ class LS:
         self,
         origin: List,
         destination: List,
-        via: Optional[List[Tuple]] = None,
+        via: Optional[List[Via]] = None,
         origin_place_options: Optional[PlaceOptions] = None,
+        origin_waypoint_options: Optional[WayPointOptions] = None,
         destination_place_options: Optional[PlaceOptions] = None,
-        via_place_options: Optional[PlaceOptions] = None,
         destination_waypoint_options: Optional[WayPointOptions] = None,
-        via_waypoint_options: Optional[WayPointOptions] = None,
         departure_time: Optional[datetime] = None,
         routing_mode: str = "fast",
         alternatives: int = 0,
@@ -611,15 +835,14 @@ class LS:
 
         :param origin: A list of ``latitude`` and ``longitude`` of origin point of route.
         :param destination: A list of ``latitude`` and ``longitude`` of destination point of route.
-        :param via: A list of tuples of ``latitude`` and ``longitude`` of via points.
+        :param via: A list of :class:`Via` objects.
         :param origin_place_options: :class:`PlaceOptions` optinal place options for ``origin``.
+        :param origin_waypoint_options: :class:`WayPointOptions` optional waypoint options
+            for ``origin``.
         :param destination_place_options: :class:`PlaceOptions` optinal place options
             for ``destination``.
-        :param via_place_options: :class:`PlaceOptions` optinal place options for ``via``.
         :param destination_waypoint_options: :class:`WayPointOptions` optional waypoint options
             for ``destination``.
-        :param via_waypoint_options: :class:`WayPointOptions` optional waypoint options for
-            ``via``.
         :param departure_time: :class:`datetime.datetime` object.
         :param routing_mode: A string to represent routing mode.
         :param alternatives: Number of alternative routes to return aside from the optimal route.
@@ -644,10 +867,9 @@ class LS:
             destination=destination,
             via=via,
             origin_place_options=origin_place_options,
+            origin_waypoint_options=origin_waypoint_options,
             destination_place_options=destination_place_options,
-            via_place_options=via_place_options,
             destination_waypoint_options=destination_waypoint_options,
-            via_waypoint_options=via_waypoint_options,
             departure_time=departure_time,
             routing_mode=routing_mode,
             alternatives=alternatives,
@@ -665,7 +887,11 @@ class LS:
         self,
         origins: List[Dict],
         region_definition: Union[
-            CircleRegion, BoundingBoxRegion, PolygonRegion, AutoCircleRegion, WorldRegion
+            CircleRegion,
+            BoundingBoxRegion,
+            PolygonRegion,
+            AutoCircleRegion,
+            WorldRegion,
         ],
         async_req: bool = False,
         destinations: Optional[List[Dict]] = None,
