@@ -2,38 +2,53 @@
 # SPDX-License-Identifier: Apache-2.0
 
 import json
-import os
-from datetime import datetime
+from datetime import datetime, timedelta
 
+import pandas as pd
 import pytest
 import pytz
-from geojson import FeatureCollection
+from geojson import FeatureCollection, Point
 
 from here_location_services import LS
+from here_location_services.config.autosuggest_config import POLITICAL_VIEW, SHOW, SearchCircle
+from here_location_services.config.base_config import (
+    ROUTING_MODE,
+    SHIPPED_HAZARDOUS_GOODS,
+    PlaceOptions,
+    Truck,
+    WayPointOptions,
+)
+from here_location_services.config.dest_weather_config import (
+    DEST_WEATHER_PRODUCT,
+    DEST_WEATHER_UNITS,
+    WEATHER_SEVERITY,
+    WEATHER_TYPE,
+)
+from here_location_services.config.isoline_routing_config import (
+    ISOLINE_ROUTING_AVOID_FEATURES,
+    ISOLINE_ROUTING_TRANSPORT_MODE,
+    RANGE_TYPE,
+)
 from here_location_services.config.matrix_routing_config import (
     AVOID_FEATURES,
     MATRIX_ATTRIBUTES,
     PROFILE,
-    SHIPPED_HAZARDOUS_GOODS,
     AutoCircleRegion,
     AvoidBoundingBox,
     BoundingBoxRegion,
     CircleRegion,
     PolygonRegion,
-    Truck,
     WorldRegion,
 )
 from here_location_services.config.routing_config import AVOID_FEATURES as ROUTING_AVOID_FEATURES
 from here_location_services.config.routing_config import (
     ROUTE_COURSE,
     ROUTE_MATCH_SIDEOF_STREET,
-    ROUTING_MODE,
     ROUTING_RETURN,
     ROUTING_SPANS,
     ROUTING_TRANSPORT_MODE,
-    PlaceOptions,
     Scooter,
-    WayPointOptions,
+    Via,
 )
 from here_location_services.config.search_config import PLACES_CATEGORIES
 from here_location_services.exceptions import ApiError
@@ -43,9 +58,155 @@ from here_location_services.utils import get_apikey
 LS_API_KEY = get_apikey()
 
 
+@pytest.mark.skipif(not LS_API_KEY, reason="No api key found.")
+def test_ls_weather_alerts():
+    """Test weather alerts endpoint of destination weather api."""
+    ls = LS(api_key=LS_API_KEY)
+    resp = ls.get_weather_alerts(
+        geometry=Point(coordinates=[15.256, 23.456]),
+        start_time=datetime.now(),
+        width=3000,
+    )
+    assert resp
+
+    resp2 = ls.get_weather_alerts(
+        geometry=Point(coordinates=[15.256, 23.456]),
+        start_time=datetime.now(),
+        weather_type=WEATHER_TYPE.ice,
+        weather_severity=WEATHER_SEVERITY.high,
+        country="US",
+        end_time=datetime.now() + timedelta(days=7),
+    )
+    assert resp2
+
+    with pytest.raises(ApiError):
+        ls2 = LS(api_key="dummy")
+        ls2.get_weather_alerts(
+            geometry=Point(coordinates=[15.256, 23.456]),
+            start_time=datetime.now(),
+        )
+
+
+@pytest.mark.skipif(not LS_API_KEY, reason="No api key found.")
+def test_ls_dest_weather():
+    """Test destination weather api."""
+    ls = LS(api_key=LS_API_KEY)
+    resp = ls.get_dest_weather(products=[DEST_WEATHER_PRODUCT.observation], query="Chicago")
+    assert resp.places
+    assert resp.places[0]["observations"]
+
+    resp2 = ls.get_dest_weather(
+        products=[DEST_WEATHER_PRODUCT.forecastHourly],
+        query="Chicago",
+        units=DEST_WEATHER_UNITS.imperial,
+    )
+    assert resp2.places
+    assert resp2.places[0]["hourlyforecasts"]
+
+    resp3 = ls.get_dest_weather(
+        products=[DEST_WEATHER_PRODUCT.forecast7days], at=["-13.163068,-72.545128"]
+    )
+    assert resp3.places
+    assert resp3.places[0]["extendedDailyforecasts"]
+
+    resp4 = ls.get_dest_weather(
+        products=[DEST_WEATHER_PRODUCT.forecast7daysSimple, DEST_WEATHER_PRODUCT.observation],
+        zipcode="10025",
+        one_observation=True,
+    )
+    assert resp4.places
+    assert resp4.places[0]["observations"]
+
+    resp5 = ls.get_dest_weather(
+        products=[DEST_WEATHER_PRODUCT.forecast7daysSimple, DEST_WEATHER_PRODUCT.observation],
+        zipcode="10025",
+        at=["-13.163068,-72.545128"],
+        one_observation=True,
+    )
+    assert resp5.places
+    assert resp5.places[0]["observations"]
+
+    with pytest.raises(ValueError):
+        ls.get_dest_weather(products=[DEST_WEATHER_PRODUCT.forecast7days])
+
+    with pytest.raises(ValueError):
+        ls.get_dest_weather(
+            products=[DEST_WEATHER_PRODUCT.forecast7days], query="Chicago", one_observation=True
+        )
+
+    with pytest.raises(ValueError):
+        ls.get_dest_weather(
+            products=[DEST_WEATHER_PRODUCT.forecast7days], query="Chicago", one_observation=True
+        )
+
+    with pytest.raises(ApiError):
+        ls2 = LS(api_key="dummy")
+        ls2.get_dest_weather(products=[DEST_WEATHER_PRODUCT.observation], query="Chicago")
+
+
+@pytest.mark.skipif(not LS_API_KEY, reason="No api key found.")
+def test_ls_autosuggest():
+    """Test autosuggest api."""
+    ls = LS(api_key=LS_API_KEY)
+    resp = ls.autosuggest(query="bar", limit=5, at=["-13.163068,-72.545128"], in_country=["USA"])
+    assert resp.items
+    assert len(resp.items) <= 5
+
+    search_in_circle1 = SearchCircle(lat=52.53, lng=13.38, radius="10000")
+    search_in_bbox1 = ("13.08836", "52.33812", "13.761", "52.6755")
+
+    resp3 = ls.autosuggest(query="bar", limit=5, search_in_circle=search_in_circle1, lang=["en"])
+    assert resp3.items
+    assert len(resp3.items) <= 5
+
+    resp4 = ls.autosuggest(
+        query="res",
+        limit=5,
+        search_in_bbox=search_in_bbox1,
+        terms_limit=3,
+        show=[SHOW.phonemes],
+        political_view=POLITICAL_VIEW.RUS,
+    )
+    assert resp4.items
+    assert len(resp4.items) <= 5
+    assert len(resp4.queryTerms) == 3
+
+    for item in resp4.items:
+        if item["resultType"] == "place":
+            assert item["politicalView"]
+            assert item["phonemes"]
+
+    with pytest.raises(ValueError):
+        ls.autosuggest(
+            query="res",
+        )
+
+    with pytest.raises(ValueError):
+        ls.autosuggest(
+            query="res",
+            search_in_bbox=search_in_bbox1,
+            search_in_circle=search_in_circle1,
+        )
+
+    with pytest.raises(ValueError):
+        ls.autosuggest(
+            query="res",
+            at=["-13.163068,-72.545128"],
+            search_in_bbox=search_in_bbox1,
+            search_in_circle=search_in_circle1,
+        )
+
+    with pytest.raises(ApiError):
+        ls2 = LS(api_key="dummy")
+        ls2.autosuggest(
+            query="res",
+            at=["-13.163068,-72.545128"],
+        )
+
+
 @pytest.mark.skipif(not LS_API_KEY, reason="No api key found.")
 def test_ls_geocoding():
-    """Test geocoding api. """
+    """Test geocoding api."""
     address = "200 S Mathilda Sunnyvale CA"
     ls = LS(api_key=LS_API_KEY)
     resp = ls.geocode(query=address, limit=2)
@@ -101,45 +262,87 @@ def test_ls_reverse_geocoding_exception():
 def test_isonline_routing():
     """Test isonline routing api."""
     ls = LS(api_key=LS_API_KEY)
+    place_options = PlaceOptions(
+        course=ROUTE_COURSE.west,
+        sideof_street_hint=[52.512149, 13.304076],
+        match_sideof_street=ROUTE_MATCH_SIDEOF_STREET.always,
+        radius=10,
+        min_course_distance=10,
+    )
+    assert json.loads(place_options.__str__()) == {
+        "course": 270,
+        "sideOfStreetHint": "52.512149,13.304076",
+        "matchSideOfStreet": "always",
+        "namehint": None,
+        "radius": 10,
+        "minCourseDistance": 10,
+    }
+    origin_waypoint_options = WayPointOptions(stop_duration=0)
+
     result = ls.calculate_isoline(
-        start=[52.5, 13.4],
-        range="900",
-        range_type="time",
-        mode="fastest;car;",
-        departure="2020-05-04T17:00:00+02",
+        origin=[52.5, 13.4],
+        range="1000,3000",
+        range_type=RANGE_TYPE.time,
+        transport_mode=ISOLINE_ROUTING_TRANSPORT_MODE.car,
+        departure_time=datetime.now(),
+        truck=Truck(
+            shipped_hazardous_goods=[SHIPPED_HAZARDOUS_GOODS.explosive],
+            gross_weight=100,
+            weight_per_axle=10,
+            height=10,
+            width=10,
+            length=10,
+            tunnel_category="B",
+            axle_count=4,
+        ),
+        shape_max_points=100,
+        avoid_features=[ISOLINE_ROUTING_AVOID_FEATURES.tollRoad],
+        origin_place_options=place_options,
+        origin_waypoint_options=origin_waypoint_options,
     )
 
-    coordinates = result.isoline[0]["component"][0]["shape"]
-    assert coordinates[0]
+    assert result.isolines
+    assert result.departure
+    coordinates = result.isolines[0]["polygons"][0]["outer"]
+    assert coordinates
     geo_json = result.to_geojson()
-    assert geo_json.type == "Feature"
-    assert geo_json.geometry.type == "Polygon"
+    assert geo_json.type == "FeatureCollection"
 
+    destination_waypoint_options = WayPointOptions(stop_duration=0)
     result2 = ls.calculate_isoline(
-        destination=[52.5, 13.4],
-        range="900",
-        range_type="time",
-        mode="fastest;car;",
-        arrival="2020-05-04T17:00:00+02",
+        destination=[52.51578, 13.37749],
+        range="600",
+        range_type=RANGE_TYPE.time,
+        transport_mode=ISOLINE_ROUTING_TRANSPORT_MODE.car,
+        destination_place_options=place_options,
+        destination_waypoint_options=destination_waypoint_options,
     )
-    coordinates = result2.isoline[0]["component"][0]["shape"]
-    assert coordinates[0]
+    assert result2.isolines
+    assert result2.arrival
 
     with pytest.raises(ValueError):
         ls.calculate_isoline(
-            start=[52.5, 13.4],
+            destination=[82.8628, 135.00],
+            range="3000",
+            range_type=RANGE_TYPE.distance,
+            transport_mode=ISOLINE_ROUTING_TRANSPORT_MODE.car,
+            arrival_time=datetime.now(),
+        )
+    with pytest.raises(ValueError):
+        ls.calculate_isoline(
+            origin=[52.5, 13.4],
             range="900",
-            range_type="time",
-            mode="fastest;car;",
+            range_type=RANGE_TYPE.time,
+            transport_mode=ISOLINE_ROUTING_TRANSPORT_MODE.car,
             destination=[52.5, 13.4],
         )
     with pytest.raises(ApiError):
         ls2 = LS(api_key="dummy")
         ls2.calculate_isoline(
-            start=[52.5, 13.4],
+            origin=[52.5, 13.4],
             range="900",
-            range_type="time",
-            mode="fastest;car;",
+            range_type=RANGE_TYPE.time,
+            transport_mode=ISOLINE_ROUTING_TRANSPORT_MODE.car,
         )
 
 
@@ -150,23 +353,23 @@ def test_isonline_routing_exception():
     with pytest.raises(ValueError):
         ls.calculate_isoline(
             range="900",
-            range_type="time",
-            mode="fastest;car;",
+            range_type=RANGE_TYPE.time,
+            transport_mode=ISOLINE_ROUTING_TRANSPORT_MODE.car,
         )
     with pytest.raises(ValueError):
         ls.calculate_isoline(
             range="900",
-            range_type="time",
-            mode="fastest;car;",
-            arrival="2020-05-04T17:00:00+02",
-            start=[52.5, 13.4],
+            range_type=RANGE_TYPE.time,
+            transport_mode=ISOLINE_ROUTING_TRANSPORT_MODE.car,
+            arrival_time=datetime.now(),
+            origin=[52.5, 13.4],
         )
     with pytest.raises(ValueError):
         ls.calculate_isoline(
             range="900",
-            range_type="time",
-            mode="fastest;car;",
-            departure="2020-05-04T17:00:00+02",
+            range_type=RANGE_TYPE.time,
+            transport_mode=ISOLINE_ROUTING_TRANSPORT_MODE.car,
+            departure_time=datetime.now(),
             destination=[52.5, 13.4],
         )
 
@@ -229,7 +432,7 @@ def test_ls_browse():
         categories=[PLACES_CATEGORIES.restaurant],
         lang="en",
     )
-    assert len(result2.items) == 10
+    assert len(result2.items) <= 10
 
     result3 = ls.browse(
         center=[19.1663, 72.8526],
@@ -238,7 +441,7 @@ def test_ls_browse():
         categories=[PLACES_CATEGORIES.restaurant],
         lang="en",
     )
-    assert len(result3.items) == 13
+    assert len(result3.items) <= 10
 
     with pytest.raises(ApiError):
         ls2 = LS(api_key="dummy")
@@ -270,26 +473,18 @@ def test_ls_lookup():
         ls2.lookup(location_id="here:pds:place:276u0vhj-b0bace6448ae4b0fbc1d5e323998a7d2")
 
 
-@pytest.mark.skipif(not LS_API_KEY, reason="No api key found.")
-def test_credentials_exception():
-    """Test exception if environment variable ``LS_API_KEY`` is not present."""
-    api_key = os.environ.get("LS_API_KEY")
-    del os.environ["LS_API_KEY"]
-    with pytest.raises(Exception):
-        _ = LS()
-    os.environ["LS_API_KEY"] = api_key
-
-
 @pytest.mark.skipif(not LS_API_KEY, reason="No api key found.")
 def test_car_route():
     """Test routing API for car route."""
     ls = LS(api_key=LS_API_KEY)
     avoid_areas = [AvoidBoundingBox(68.1766451354, 7.96553477623, 97.4025614766, 35.4940095078)]
     avoid_features = [ROUTING_AVOID_FEATURES.tollRoad]
+    via1 = Via(lat=52.52426, lng=13.43000)
+    via2 = Via(lat=52.52624, lng=13.44012)
     result = ls.car_route(
         origin=[52.51375, 13.42462],
         destination=[52.52332, 13.42800],
-        via=[(52.52426, 13.43000)],
+        via=[via1, via2],
         return_results=[ROUTING_RETURN.polyline, ROUTING_RETURN.elevation],
         departure_time=datetime.now(),
         spans=[ROUTING_SPANS.names],
@@ -329,22 +524,48 @@ def test_car_route_extra_options():
         "minCourseDistance": 10,
     }
     via_waypoint_options = WayPointOptions(stop_duration=0, pass_through=True)
-    assert json.loads(via_waypoint_options.__str__()) == {"stopDuration": 0, "passThrough": True}
+    assert json.loads(via_waypoint_options.__str__()) == {
+        "stopDuration": 0,
+        "passThrough": True,
+    }
     dest_waypoint_options = WayPointOptions(stop_duration=10, pass_through=False)
+    via1 = Via(
+        lat=52.52426,
+        lng=13.43000,
+        place_options=place_options,
+        waypoint_options=via_waypoint_options,
+    )
+    via2 = Via(lat=52.52426, lng=13.43000)
+    via3 = Via(
+        lat=52.52426,
+        lng=13.43000,
+        place_options=place_options,
+        waypoint_options=via_waypoint_options,
+    )
     ls = LS(api_key=LS_API_KEY)
-    _ = ls.car_route(
+    resp = ls.car_route(
         origin=[52.51375, 13.42462],
         destination=[52.52332, 13.42800],
-        via=[(52.52426, 13.43000), (52.123, 13.22), (52.343, 13.444)],
+        via=[via1, via2, via3],
         origin_place_options=place_options,
         destination_place_options=place_options,
-        via_place_options=place_options,
-        via_waypoint_options=via_waypoint_options,
         destination_waypoint_options=dest_waypoint_options,
         return_results=[ROUTING_RETURN.polyline, ROUTING_RETURN.elevation],
         departure_time=datetime.now(),
         spans=[ROUTING_SPANS.names],
     )
+    resp = resp.response
+    assert len(resp["routes"][0]["sections"]) == 2
+    assert list(resp["routes"][0]["sections"][0].keys()) == [
+        "id",
+        "type",
+        "departure",
+        "arrival",
+        "polyline",
+        "spans",
+        "notices",
+        "transport",
+    ]
 
 
 @pytest.mark.skipif(not LS_API_KEY, reason="No api key found.")
@@ -353,10 +574,11 @@ def test_bicycle_route():
     ls = LS(api_key=LS_API_KEY)
     avoid_areas = [AvoidBoundingBox(68.1766451354, 7.96553477623, 97.4025614766, 35.4940095078)]
     avoid_features = [ROUTING_AVOID_FEATURES.tollRoad]
+    via = Via(lat=52.52426, lng=13.43000)
     _ = ls.bicycle_route(
         origin=[52.51375, 13.42462],
         destination=[52.52332, 13.42800],
-        via=[(52.52426, 13.43000)],
+        via=[via],
         return_results=[ROUTING_RETURN.polyline, ROUTING_RETURN.elevation],
         departure_time=datetime.now(),
         spans=[ROUTING_SPANS.names],
@@ -382,10 +604,11 @@ def test_truck_route():
     )
     avoid_areas = [AvoidBoundingBox(68.1766451354, 7.96553477623, 97.4025614766, 35.4940095078)]
     avoid_features = [ROUTING_AVOID_FEATURES.tollRoad]
+    via = Via(lat=52.52426, lng=13.43000)
     _ = ls.truck_route(
         origin=[52.51375, 13.42462],
         destination=[52.52332, 13.42800],
-        via=[(52.52426, 13.43000)],
+        via=[via],
         return_results=[ROUTING_RETURN.polyline, ROUTING_RETURN.elevation],
         departure_time=datetime.now(),
         spans=[ROUTING_SPANS.names],
@@ -401,10 +624,12 @@ def test_scooter_route():
     """Test routing API for scooter route."""
     ls = LS(api_key=LS_API_KEY)
     scooter = Scooter(allow_highway=True)
+    assert json.loads(scooter.__str__()) == {"allowHighway": True}
+    via = Via(lat=52.52426, lng=13.43000)
     _ = ls.scooter_route(
         origin=[52.51375, 13.42462],
         destination=[52.52332, 13.42800],
-        via=[(52.52426, 13.43000)],
+        via=[via],
         return_results=[ROUTING_RETURN.polyline, ROUTING_RETURN.elevation],
         departure_time=datetime.now(),
         spans=[ROUTING_SPANS.names],
@@ -417,10 +642,11 @@ def test_scooter_route():
 def test_pedestrian_route():
     """Test routing API for pedestrian route."""
     ls = LS(api_key=LS_API_KEY)
+    via = Via(lat=52.52426, lng=13.43000)
     _ = ls.pedestrian_route(
         origin=[52.51375, 13.42462],
         destination=[52.52332, 13.42800],
-        via=[(52.52426, 13.43000)],
+        via=[via],
         return_results=[ROUTING_RETURN.polyline, ROUTING_RETURN.elevation],
         departure_time=datetime.now(),
         spans=[ROUTING_SPANS.names],
@@ -527,6 +753,8 @@ def test_matrix_route():
     assert mat2["numDestinations"] == 3
     with pytest.raises(NotImplementedError):
         result2.to_geojson()
+    assert isinstance(result2.to_distnaces_matrix(), pd.DataFrame)
+    assert isinstance(result2.to_travel_times_matrix(), pd.DataFrame)
 
 
 @pytest.mark.skipif(not LS_API_KEY, reason="No api key found.")
@@ -589,7 +817,10 @@ def test_matrix_routing_config():
     }
 
     poly = PolygonRegion(outer=[1, 1, 1, 1, 1, 1])
-    assert json.loads(poly.__str__()) == {"type": "polygon", "outer": [1, 1, 1, 1, 1, 1]}
+    assert json.loads(poly.__str__()) == {
+        "type": "polygon",
+        "outer": [1, 1, 1, 1, 1, 1],
+    }
 
     autocircle = AutoCircleRegion(margin=100)
     assert json.loads(autocircle.__str__()) == {"type": "autoCircle", "margin": 100}
